/*
    Title:    UART library
    Author:   Volker Oth
    Date:     5/1999
    Purpose:  Sending a block of data to uart.
    needed
    Software: AVR-GCC to compile, AVA to assemble and link
    needed
    Hardware: ATS90S8515 on STK200 board
    Note:     To contact me, mail to
                  volkeroth@gmx.de
              You might find more AVR related stuff at my homepage:
                  http://members.xoom.com/volkeroth
*/

#include <avr/io.h>
#include <avr/signal.h>
#include <avr/interrupt.h>
#include "global.h"
//#include "timer.h"
#include "uart.h"
//#include "atmega_compat.h"



/* uart globals */
volatile u08 uart_txd_buf_cnt;
volatile u08 uart_rxd_buf_cnt;
u08 *uart_txd_in_ptr, *uart_txd_out_ptr;
u08 *uart_rxd_in_ptr, *uart_rxd_out_ptr;
u08 UART_CLR[] = {ESC, '[','H', ESC, '[', '2', 'J',0};
u08 UART_NL[] = {0x0d,0x0a,0};
u08 uart_txd_buffer[UART_BUF_SIZE];
u08 uart_rxd_buffer[UART_BUF_SIZE];

char watchfor;

char * readLine();
unsigned char watchEnable,lineEnable;
signal signalfunc;
sig_newline gotlinefunc;

void uart_setSignalHandler(signal s)
{
	signalfunc = s;
}

void uart_setNewlineSignalHandler(sig_newline s)
{
	gotlinefunc = s;
}

void uart_setNewlineSignalEnable(unsigned char r)
{
	lineEnable = r;
}



void uart_setWatchEnable(unsigned char t)
{
	if (t > 0) 
		watchEnable = 1;
		else watchEnable =0;
}

void uart_watchFor(char peace)
{
	watchfor = peace;
}

		
void uart_init(void)
/* initialize uart */
{
    /* enable RxD/TxD, receive complete int */
    outp((1<<RXCIE)|(1<<RXEN)|(1<<TXEN),UCSR1B);
    /* set baud rate */
    outp(51, UBRR1L);
    outp(0x00,UBRR1H);
    outp(3<<UCSZ0,UCSR1C);
    //sbi(UCSR0C,USBS);
    uart_txd_in_ptr  = uart_txd_out_ptr = uart_txd_buffer;
    uart_rxd_in_ptr  = uart_rxd_out_ptr = uart_rxd_buffer;
    uart_txd_buf_cnt = 0;
    uart_rxd_buf_cnt = 0;
	watchEnable = 0;
	lineEnable = 0;
}



SIGNAL (SIG_UART1_DATA)
/* signal handler for uart data buffer empty interrupt */
{
		//outp(0xff,PORTC);
	//	sbi(PORTC,4);
    if (uart_txd_buf_cnt > 0) {
			//outp(0x00,PORTC);
        outp(*uart_txd_out_ptr, UDR1);           /* write byte to data buffer */
				//outp(2,PORTC);
        if (++uart_txd_out_ptr >= uart_txd_buffer + UART_BUF_SIZE) /* Pointer wrapping */
            uart_txd_out_ptr = uart_txd_buffer;
        if(--uart_txd_buf_cnt == 0)             /* if buffer is empty: */
				{
     //       outp(UCR, inp(UCR)&(!(1<<UDRIE)));                    /* disable UDRIE int */
     cbi(UCSR1B,UDRIE);
		// outp(4,PORTC);
		 	}

    }
}


SIGNAL (SIG_UART1_RECV)
/* signal handler for receive complete interrupt */
{
	int tosignal=0;
	int linesignal=0;
    *uart_rxd_in_ptr = inp(UDR1);            /* read byte from receive register */
	
	if (*(uart_rxd_in_ptr) == watchfor && watchEnable) {
		tosignal = 1;
		}
	if ( *(uart_rxd_in_ptr) == '\r' && lineEnable) linesignal =1;
	
    uart_rxd_buf_cnt++;
    if (++uart_rxd_in_ptr >= uart_rxd_buffer + UART_BUF_SIZE) /* Pointer wrapping */
        uart_rxd_in_ptr = uart_rxd_buffer;
		
		
		if (tosignal) signalfunc(UART_STAT_WATCH);
		if (linesignal) gotlinefunc(readLine());
		
}

char * readLine()
{
	
	//cbi(UCSR1B,UDRIE); // turn all that shit off for now
	
	char * ret = (char *)malloc((uart_rxd_buf_cnt+10) * sizeof(char));
	int x=0;
	char val_iter;
	val_iter = uart_getchar();
	while (val_iter != -1)	{
		*(ret  +(x++) ) = val_iter;
		val_iter = uart_getchar();
	}
	*(ret + x-1) = '\0';
	
	
	return ret;
	
}

s16 uart_getchar(void)
{
    u08 c;

    if (uart_rxd_buf_cnt>0) {
        cli();
        uart_rxd_buf_cnt--;
        c = *uart_rxd_out_ptr;              /* get character from buffer */
        if (++uart_rxd_out_ptr >= uart_rxd_buffer + UART_BUF_SIZE) /* pointer wrapping */
            uart_rxd_out_ptr = uart_rxd_buffer;
        sei();
        return c;
    }
    else
        return -1;                          /* buffer is empty */
}


bool uart_putchar(u08 c)
{
		char old;

		//uart_txd_buf_cnt=0;
/*
						while (!(inp(UCSR1A) & (1<<UDRE)))
						{}
						if (uart_txd_buf_cnt<10) {
						outp(0xff,PORTC);
						}


						outp(c, UDR1);
						*/



    if (uart_txd_buf_cnt<UART_BUF_SIZE) {
        cli();
        uart_txd_buf_cnt++;
        *uart_txd_in_ptr = c;               /* put character into buffer */
	//			sbi(PORTC,1);
        if (++uart_txd_in_ptr >= uart_txd_buffer + UART_BUF_SIZE) /* pointer wrapping */
            uart_txd_in_ptr = uart_txd_buffer;




      //  outp(UCR, inp(UCR)|((1<<UDRIE)));


        sbi(UCSR1B, UDRIE);                    /* enable UDRIE int */
				//old = inp(UCSR1B);
			//	outp(old | (1<<UDRIE),UCSR1B);

        sei();
        return 1;
    }
    else
        return 0;                           /* buffer is full */
}


bool uart_putstr(u08 s[])
{
    char *c = s;

    while (*c)
        if (uart_putchar(*c))
            c++;

	//free(c);
    return 1;
}


void uart_clr(void)
/* Send a 'clear screen' to a VT100 terminal */
{
    uart_putstr(UART_CLR);
}


void uart_nl(void)
/* Send a 'new line' */
{
    uart_putstr(UART_NL);
}

